package com.sunlabs.archsim.fleet.ships.accumulator;
import com.sunlabs.archsim.fleet.common.OutputFunction;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;

/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

class AckAction extends Action {


	/* */
	private Terminal internalIn;
	private Terminal ackInternalIn;

		
	public AckAction(String name){
		super(name);
                // make an AckFunction - send an ack to the ACK terminal
		OutputFunction ackFunction= new AckFunction("Done");
		// associate done wioth the external done output
		outputFunctions.put(AccumulatorDefaults.ACK_INTERNAL_IN, ackFunction);
	}
	
	
	/**
	 * Attach a terminal.
	 * Attach only terminals that are needed for this action, ignore the rest.
	 * Self check ensures that all needed terminals are attached
	 */
	public void attach(Terminal t) {
		// get the bas ename for check
		String termName= t.getBaseName();
		// remember whether the terminal was of a bad type
		boolean badType= false;
		// remember whether the same name been added twice?
		boolean twice= false;
                
		if (termName.equals(AccumulatorDefaults.ACK_INTERNAL_IN)) { 
			if (ackInternalIn == null) {
				if (t instanceof OutputTerminal) {
					t.attachAction(this);
        				ackInternalIn= (OutputTerminal)t;
					terminals.add(t);
				} 
				else { 
					badType= true;
				}
			}
			else {
				twice= true;
			}
		}
		else if (termName.equals(AccumulatorDefaults.INTERNAL_IN)) { 
			if (internalIn == null) {
				if (t instanceof InputTerminal) {
					t.attachAction(this);
					numInputs++;
                                        internalIn= (InputTerminal)t;
					terminals.add(t);
				} 
				else { 
					badType= true;
				}
			}
			else {
				twice= true;
			}
		}
		else {
			// unknown terminal
			director.error("Terminal " 
					+ t.getName() 
					+ " cannot be attached to OutputAction "
					+ getName() + ", id= " + getID()
					+ "; terminal name not recognized");
		}
		
		// check for errors
		if (badType) {
			// the terminal was of a wrong type
			director.fatalError("AckAction, " + getName() + ", " + getID() +  
					": Terminal " + t.getName() + " is of a bad type");
		}
		if (twice) {
			// this was the second addition of the same name
			director.fatalError("AckAction, " + getName() + ", " + getID() 
					+ ": Terminal " + t.getName() + " cannot be attached twice");
		}
			
		
	} // attach
	
	
	/** self check - are all the terminals attached, run the self check from superclass */
	public boolean selfCheck() {
		boolean OK= true;
//		boolean check;
                /* TODO
                check= (doneTerm != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal "
					+ StrideDefaults.DONE);
		}

		
		// also run the self check on the superclass
		check= super.selfCheck();
		OK= OK && check;
                */		
		return OK;
	} // selfCheck

        /** return the internalIn terminal */
	public Terminal getIn() {
                return internalIn;
        }
        
        
} // class OutputAction
