/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Mar 24, 2005
 *
 */
package com.sunlabs.archsim.tests.branch;

import java.util.Iterator;
import java.util.LinkedList;


import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.Function;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;

/**
 * @author ib27688
 *
 */
public class BranchTestSink extends Action {
	
	private ActionFired function= new ActionFired();
	
	// the list to store values received
	private LinkedList<Data> results;
	
	public BranchTestSink(String name) {
		super(name);
		// extract the number at the end of the name 
		// the name is neede to connect the output function to its port
		String n= "";
		int i= name.length()-1;
		while (i >=0 && name.charAt(i) >= '0' && name.charAt(i) <= '9') {
			n= name.substring(i,i+1) + n;
			i--;
		}
		
		// create the list for storing results
		results= new LinkedList<Data>();
		
		outputFunctions.put(BranchTestGlobal.SINK_ACK_IN+n,
							new ActionFired());
	} // BranchTestSink
	
	public BranchTestSink(String name, CompositeEntity g) {
		this(name);
		g.addMember(this);
	}
		
	
	/** set the sink delay */
	public void setDelay(Delay d) {
		function.setDelay(d);
	}
	
	/** Vacuous function used to acknowldge that the action has fired */ 
	private class ActionFired extends Function {
		
		public ActionFired() {
			super("Vacous");
			functionDelay= BranchTestGlobal.SINK_DELAY; 
		}
	
		public Object value() {
			Iterator ti= terminals.iterator();
			Terminal t= null;
			boolean done= false;
			while (!done && ti.hasNext()) {
				t= (Terminal)(ti.next());
				if (t instanceof InputTerminal) { // (t.getBaseName() == "dataIn") {
					done= true;
				}
			}
			if (t != null) {
				Payload p= (Payload)t.getData();
				Data d= p.data;				
				results.addLast(d);
				if (BranchTestGlobal.sinkDisplayReceived ) {
					director.journal("Sink " + BranchTestSink.this.getName() 
							+ ", id= " + BranchTestSink.this.getID()
							+ ", received " + t.getData());
				} // if journal

			} // found the terminal
			
			return new Data();
		} // value
	} // AcknowledgeData
	
	// accessor for results
	LinkedList<Data> getResults() {
		return results;
	} // getResultts
	
} // BranchTestSink
